{*******************************************************}
{                 TpsvShine component                   }
{                     version 3.1                       }
{ Author:                                               }
{ Serhiy Perevoznyk                                     }
{ serge_perevoznyk@hotmail.com                          }
{                                                       }
{     Use, modification and distribution is allowed     }
{without limitation, warranty, or liability of any kind.}
{                                                       }
{ Contributor(s):                                       }
{ Eduard (mattsoft@zeelandnet.nl)                       }
{ Goncharov Sergey (goncharov@tsure.ru)                 }
{                                                       }
{*******************************************************}


unit psvShine;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ExtCtrls;

type
  TShinePos = (spLeftTop, spLeftBottom, spRightTop, spRightBottom, spMousePos);

  TpsvShine = class(TComponent)
  private
    { Private declarations }
    FColor    : TColor;
    AnimTimer : TTimer;
    FrameNR   : integer;
    tmpBmp    : TBitmap;
    FControl  : TControl;
    FPos      : TShinePos;
    LPoint    : Tpoint;
    procedure SetColor_(Value: TColor);
    procedure SetActive(Value : boolean);
    function  GetActive : boolean;
  protected
    { Protected declarations }
    procedure TimerTick(Sender : TObject);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetControl(aValue : TControl);
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor  Destroy; override;
    procedure   Execute;
    procedure   Stop;

  published
    { Published declarations }
    property Color   : TColor read FColor write SetColor_;
    property Control : TControl read FControl write SetControl;
    property Position : TShinePos read FPos write FPos;
    property Active : boolean read GetActive write SetActive stored false;
  end;

procedure Register;

implementation

{$R psvShine.RES}

procedure Register;
begin
  RegisterComponents('Additional', [TpsvShine]);
end;

constructor TpsvShine.Create(AOwner : TComponent);
begin
 inherited;
 AnimTimer:=TTimer.Create(Self);
 AnimTimer.Interval:=200;
 AnimTimer.OnTimer:=TimerTick;
 AnimTimer.Enabled:=False;
 FrameNr := 0;
 Color := clYellow;
end;

destructor TpsvShine.Destroy;
begin
 AnimTimer.Enabled := false;
 AnimTimer.Free;
 inherited;
end;

procedure TpsvShine.SetColor_(Value: TColor);
begin
  FColor := Value;
end;

procedure TpsvShine.SetActive(Value : boolean);
begin
  Execute;
end;

function TpsvShine.GetActive : boolean;
begin
  Result := AnimTimer.Enabled;
end;

procedure TpsvShine.Execute;
begin
  if AnimTimer.Enabled then
   begin
     Exit;
   end;
  if not Assigned(Owner) then
   Exit;

  if ( not ( Owner is TCustomForm) ) then
   Exit;

  if Assigned(FControl) then
   begin
      case FPos of
        spLeftTop     : LPoint := FControl.ClientToScreen(point(0,0));
        spLeftBottom  : LPoint := FControl.ClientToScreen(point(0,FControl.Height));
        spRightTop    : LPoint := FControl.ClientToScreen(point(FControl.Width,0));
        spRightBottom : LPoint := FControl.ClientToScreen(point(FControl.Width,FControl.Height));
        spMousePos    : GetCursorPos(LPoint); //FControl.ClientToScreen(point());
      end;
      LPoint := TCustomForm(Owner).ScreenToClient(LPoint);
     AnimTimer.Enabled := true;
   end;
end;

procedure TpsvShine.TimerTick(Sender : TObject);
var
 R : TRect;
 Style: Longint;
begin
  inc(FrameNr);
  Style:= GetWindowLong(TCustomForm(Owner).Handle, GWl_Style);
  Style:= Style and (not WS_ClipChildren);
  SetWindowLong(TCustomForm(Owner).Handle, GWl_Style, Style);

  R := Rect(LPoint.X - 10, LPoint.Y - 14,  LPoint.X + 10, LPoint.y + 14);
  InvalidateRect(TCustomForm(Owner).Handle, @R, true);
  TCustomForm(Owner).Update;


  if FrameNr = 9 then
   begin
    AnimTimer.Enabled := False;
    FrameNr := 0;
    Style:= Style or WS_ClipChildren;
    SetWindowLong(TCustomForm(Owner).Handle, GWl_Style, Style);
    Exit;
   end;

  tmpBmp := TBitMap.Create;
  tmpBmp.Transparent := true;
  tmpBmp.Width := 20;
  tmpBmp.Height:= 28;
  tmpBmp.Handle := LoadBitmap(hInstance, PChar('SPARK_'+IntToStr(FrameNr)));
  TCustomForm(Owner).Canvas.Brush.Color := FColor;

  BitBlt(TCustomForm(Owner).Canvas.Handle,LPoint.X - 10, LPoint.Y - 14, 20,28,tmpBmp.Canvas.Handle,0,0, srcPaint);

  Style:= Style or WS_ClipChildren;
  SetWindowLong(TCustomForm(Owner).Handle, GWl_Style, Style);

  tmpBmp.Free;
end;

procedure TpsvShine.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FControl) then
    FControl := nil;
end;

procedure TpsvShine.SetControl(aValue : TControl);
begin
  if FControl <> aValue then
  begin
    FControl := aValue;
    if aValue <> nil then
     begin
      aValue.FreeNotification(Self);
     end;
  end;
end;



procedure TpsvShine.Stop;
begin
  AnimTimer.Enabled := False;
  FrameNr := 0;
  if (Owner is TCustomForm)  then
   TCustomForm(Owner).Update;
  FControl.Repaint; 
end;

end.
